"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginManager = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const child_process_1 = require("child_process");
const _1 = require(".");
class PluginManager extends _1.BasePlugin {
    constructor() {
        super(...arguments);
        this.commands = {
            plugin: {
                usage: 'manage plugins',
                options: {
                    update: {
                        usage: 'update a plugin',
                        shortcut: 'u',
                    },
                    remove: {
                        usage: 'remove a plugin',
                        shortcut: 'r',
                    },
                    install: {
                        usage: 'install a plugin',
                        shortcut: 'i',
                    },
                    list: {
                        usage: 'list all plugin',
                        shortcut: 'l',
                    },
                    root: {
                        usage: 'get plugin root path',
                    },
                    all: {
                        usage: 'do something all',
                        shortcut: 'a',
                    },
                    npm: {
                        usage: 'npm',
                        shortcut: 'n',
                    },
                },
                lifecycleEvents: ['plugin'],
            },
        };
        this.hooks = {
            'plugin:plugin': async () => {
                if (this.options.update) {
                    await this.update();
                }
                else if (this.options.install) {
                    await this.install();
                }
                else if (this.options.list) {
                    await this.list();
                }
                else if (this.options.root) {
                    this.core.cli.log(`  - plugin root path is ${_1.getCoreBaseDir()}`);
                }
            },
        };
    }
    async update() {
        if (this.options.a) {
            console.log('update all');
        }
        else if (typeof this.options.update === 'string') {
            await this.updatePlugin(this.options.update);
        }
    }
    async list() {
        const plugins = this.getAllPlugins();
        plugins.forEach(plugin => {
            const version = this.getPluginVersion(plugin);
            this.core.cli.log(`  - ${plugin}${version ? `@${version}` : ''}`);
        });
    }
    async install() {
        if (typeof this.options.install === 'string') {
            const plugin = this.options.install.replace(/@.*$/, '');
            const dir = _1.getCoreBaseDir();
            await this.execPromise(`cd ${path_1.join(dir, '../')};${this.options.npm || 'npm'} install ${this.options.install} --production`);
            this.core.cli.log(`  - installed '${plugin}'`);
        }
        else {
            this.core.cli.log('please user plugin -i=<plugin name>');
        }
    }
    getAllPlugins() {
        const dir = _1.getCoreBaseDir();
        if (!fs_1.existsSync(dir)) {
            return [];
        }
        const files = fs_1.readdirSync(dir);
        const plugins = files.filter(file => {
            return /midway-plugin/.test(file);
        });
        return plugins;
    }
    getPluginVersion(plugin) {
        const dir = _1.getCoreBaseDir();
        const pkgJson = path_1.join(dir, plugin, 'package.json');
        let version = '';
        try {
            version = JSON.parse(fs_1.readFileSync(pkgJson).toString()).version;
        }
        catch (E) {
            /** ignore */
        }
        return version;
    }
    async updatePlugin(newPlugin) {
        const plugin = newPlugin.replace(/@.*$/, '');
        const oldVersion = this.getPluginVersion(plugin);
        const dir = _1.getCoreBaseDir();
        await this.execPromise(`cd ${dir};rm -rf ${plugin}`);
        await this.execPromise(`cd ${path_1.join(dir, '../')};${this.options.npm || 'npm'} install ${newPlugin} --production`);
        const newVersion = this.getPluginVersion(plugin);
        this.core.cli.log(`  - update '${plugin}'${oldVersion ? ` from ${oldVersion}` : ''}${newVersion ? ` to ${newVersion}` : ''}`);
    }
    execPromise(cmd) {
        return new Promise((resolve, reject) => {
            child_process_1.exec(cmd, err => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve();
            });
        });
    }
}
exports.PluginManager = PluginManager;
//# sourceMappingURL=pluginManager.js.map